@@ -21,7 +21,6 @@ that import the `os`, `io`, `path` `shutil`, and `pathlib` modules.
 
 The `setUpPyfakefs()` method binds these modules to the corresponding fake
 modules from `pyfakefs`.  Further, the `open()` built-in is bound to a fake
-`open()`.  In Python 2, built-in `file()` is similarly bound to the fake
 `open()`.
 
 It is expected that `setUpPyfakefs()` be invoked at the beginning of the
@@ -37,59 +36,102 @@ pyfakefs by simply changing their base class from `:py:class`unittest.TestCase`
 to `:py:class`pyfakefs.fake_filesystem_unittest.TestCase`.
 """
 import doctest
+import functools
 import inspect
+import linecache
 import shutil
 import sys
 import tempfile
+import tokenize
 import unittest
 import warnings
 
+import py
+import pytest
+
 from pyfakefs.deprecator import Deprecator
-from pyfakefs.fake_filesystem import set_uid, set_gid, reset_ids
-from pyfakefs.helpers import IS_PY2, IS_PYPY
+from pyfakefs.fake_filesystem import set_uid, set_gid, reset_ids, PatchMode
+from pyfakefs.helpers import IS_PYPY
+from pyfakefs.patched_packages import (
+    get_modules_to_patch, get_classes_to_patch, get_fake_module_classes
+)
 
 try:
     from importlib.machinery import ModuleSpec
 except ImportError:
     ModuleSpec = object
 
-try:
-    # python >= 3.4
-    from importlib import reload
-except ImportError:
-    try:
-        # python 3.0 - 3.3
-        from imp import reload
-    except ImportError:
-        # python 2 - reload is built-in
-        pass
+from importlib import reload
 
 from pyfakefs import fake_filesystem
 from pyfakefs import fake_filesystem_shutil
+from pyfakefs import fake_pathlib
 from pyfakefs import mox3_stubout
 from pyfakefs.extra_packages import pathlib, pathlib2, use_scandir
 
-if pathlib:
-    from pyfakefs import fake_pathlib
 
 if use_scandir:
     from pyfakefs import fake_scandir
 
-if sys.version_info < (3,):
-    import __builtin__ as builtins  # pylint: disable=import-error
-else:
-    import builtins
-
 OS_MODULE = 'nt' if sys.platform == 'win32' else 'posix'
 PATH_MODULE = 'ntpath' if sys.platform == 'win32' else 'posixpath'
-BUILTIN_MODULE = '__builtin__'
 
 
-def load_doctests(loader, tests, ignore, module,
-                  additional_skip_names=None,
-                  modules_to_reload=None,
-                  modules_to_patch=None,
-                  allow_root_user=True):  # pylint: disable=unused-argument
+def patchfs(_func=None, *,
+            additional_skip_names=None,
+            modules_to_reload=None,
+            modules_to_patch=None,
+            allow_root_user=True,
+            use_known_patches=True,
+            patch_open_code=PatchMode.OFF):
+    """Convenience decorator to use patcher with additional parameters in a
+    test function.
+
+    Usage::
+
+        @patchfs
+        def test_my_function(fs):
+            fs.create_file('foo')
+
+        @patchfs(allow_root_user=False)
+        def test_with_patcher_args(fs):
+            os.makedirs('foo/bar')
+    """
+
+    def wrap_patchfs(f):
+        @functools.wraps(f)
+        def wrapped(*args, **kwargs):
+            with Patcher(
+                    additional_skip_names=additional_skip_names,
+                    modules_to_reload=modules_to_reload,
+                    modules_to_patch=modules_to_patch,
+                    allow_root_user=allow_root_user,
+                    use_known_patches=use_known_patches,
+                    patch_open_code=patch_open_code) as p:
+                kwargs['fs'] = p.fs
+                return f(*args, **kwargs)
+
+        return wrapped
+
+    if _func:
+        if not callable(_func):
+            raise TypeError(
+                "Decorator argument not a function.\n"
+                "Did you mean `@patchfs(additional_skip_names=...)`?"
+            )
+        return wrap_patchfs(_func)
+
+    return wrap_patchfs
+
+
+def load_doctests(
+        loader, tests, ignore, module,
+        additional_skip_names=None,
+        modules_to_reload=None,
+        modules_to_patch=None,
+        allow_root_user=True,
+        use_known_patches=True,
+        patch_open_code=PatchMode.OFF):  # pylint: disable=unused-argument
     """Load the doctest tests for the specified module into unittest.
         Args:
             loader, tests, ignore : arguments passed in from `load_tests()`
@@ -101,7 +143,9 @@ def load_doctests(loader, tests, ignore, module,
     _patcher = Patcher(additional_skip_names=additional_skip_names,
                        modules_to_reload=modules_to_reload,
                        modules_to_patch=modules_to_patch,
-                       allow_root_user=allow_root_user)
+                       allow_root_user=allow_root_user,
+                       use_known_patches=use_known_patches,
+                       patch_open_code=patch_open_code)
     globs = _patcher.replace_globs(vars(module))
     tests.addTests(doctest.DocTestSuite(module,
                                         globs=globs,
@@ -110,7 +154,7 @@ def load_doctests(loader, tests, ignore, module,
     return tests
 
 
-class TestCaseMixin(object):
+class TestCaseMixin:
     """Test case mixin that automatically replaces file-system related
     modules by fake implementations.
 
@@ -162,10 +206,12 @@ class TestCaseMixin(object):
                       additional_skip_names=None,
                       modules_to_reload=None,
                       modules_to_patch=None,
-                      allow_root_user=True):
+                      allow_root_user=True,
+                      use_known_patches=True,
+                      patch_open_code=PatchMode.OFF):
         """Bind the file-related modules to the :py:class:`pyfakefs` fake file
         system instead of the real file system.  Also bind the fake `open()`
-        function, and on Python 2, the `file()` function.
+        function.
 
         Invoke this at the beginning of the `setUp()` method in your unit test
         class.
@@ -184,7 +230,9 @@ class TestCaseMixin(object):
             additional_skip_names=additional_skip_names,
             modules_to_reload=modules_to_reload,
             modules_to_patch=modules_to_patch,
-            allow_root_user=allow_root_user
+            allow_root_user=allow_root_user,
+            use_known_patches=use_known_patches,
+            patch_open_code=patch_open_code
         )
 
         self._stubber.setUp()
@@ -218,8 +266,7 @@ class TestCase(unittest.TestCase, TestCaseMixin):
     def __init__(self, methodName='runTest',
                  additional_skip_names=None,
                  modules_to_reload=None,
-                 modules_to_patch=None,
-                 allow_root_user=True):
+                 modules_to_patch=None):
         """Creates the test class instance and the patcher used to stub out
         file system related modules.
 
@@ -232,7 +279,6 @@ class TestCase(unittest.TestCase, TestCaseMixin):
         self.additional_skip_names = additional_skip_names
         self.modules_to_reload = modules_to_reload
         self.modules_to_patch = modules_to_patch
-        self.allow_root_user = allow_root_user
 
     @Deprecator('add_real_file')
     def copyRealFile(self, real_file_path, fake_file_path=None,
@@ -260,7 +306,7 @@ class TestCase(unittest.TestCase, TestCaseMixin):
           The newly created FakeFile object.
 
         Raises:
-          IOError: If the file already exists in the fake file system.
+          OSError: If the file already exists in the fake file system.
           ValueError: If deprecated argument values are specified.
 
         See:
@@ -282,7 +328,7 @@ class TestCase(unittest.TestCase, TestCaseMixin):
         pass
 
 
-class Patcher(object):
+class Patcher:
     """
     Instantiate a stub creator to bind and un-bind the file-related modules to
     the :py:mod:`pyfakefs` fake modules.
@@ -298,46 +344,82 @@ class Patcher(object):
     '''Stub nothing that is imported within these modules.
     `sys` is included to prevent `sys.path` from being stubbed with the fake
     `os.path`.
+    The `pytest` and `py` modules are used by pytest and have to access the
+    real file system.
+    The `linecache` module is used to read the test file in case of test
+    failure to get traceback information before test tear down.
+    In order to make sure that reading the test file is not faked,
+    we skip faking the module.
+    We also have to set back the cached open function in tokenize.
     '''
-    SKIPMODULES = {None, fake_filesystem, fake_filesystem_shutil, sys}
+    SKIPMODULES = {
+        None, fake_filesystem, fake_filesystem_shutil,
+        sys, linecache, tokenize, py, pytest
+    }
     assert None in SKIPMODULES, ("sys.modules contains 'None' values;"
                                  " must skip them.")
 
     IS_WINDOWS = sys.platform in ('win32', 'cygwin')
 
     SKIPNAMES = {'os', 'path', 'io', 'genericpath', OS_MODULE, PATH_MODULE}
-    if pathlib:
-        SKIPNAMES.add('pathlib')
-    if pathlib2:
-        SKIPNAMES.add('pathlib2')
 
     def __init__(self, additional_skip_names=None,
                  modules_to_reload=None, modules_to_patch=None,
-                 allow_root_user=True):
-        """For a description of the arguments, see TestCase.__init__"""
+                 allow_root_user=True, use_known_patches=True,
+                 patch_open_code=PatchMode.OFF):
+        """
+        Args:
+            additional_skip_names: names of modules inside of which no module
+                replacement shall be performed, in addition to the names in
+                :py:attr:`fake_filesystem_unittest.Patcher.SKIPNAMES`.
+                Instead of the module names, the modules themselves
+                may be used.
+            modules_to_reload: A list of modules that need to be reloaded
+                to be patched dynamically; may be needed if the module
+                imports file system modules under an alias
+
+                .. caution:: Reloading modules may have unwanted side effects.
+            modules_to_patch: A dictionary of fake modules mapped to the
+                fully qualified patched module names. Can be used to add
+                patching of modules not provided by `pyfakefs`.
+            allow_root_user: If True (default), if the test is run as root
+                user, the user in the fake file system is also considered a
+                root user, otherwise it is always considered a regular user.
+            use_known_patches: If True (the default), some patches for commonly
+                used packages are applied which make them usable with pyfakefs.
+            patch_open_code: If True, `io.open_code` is patched. The default
+                is not to patch it, as it mostly is used to load compiled
+                modules that are not in the fake file system.
+        """
 
         if not allow_root_user:
             # set non-root IDs even if the real user is root
             set_uid(1)
             set_gid(1)
 
-        self._skipNames = self.SKIPNAMES.copy()
+        self._skip_names = self.SKIPNAMES.copy()
         # save the original open function for use in pytest plugin
         self.original_open = open
         self.fake_open = None
+        self.patch_open_code = patch_open_code
 
         if additional_skip_names is not None:
             skip_names = [m.__name__ if inspect.ismodule(m) else m
                           for m in additional_skip_names]
-            self._skipNames.update(skip_names)
+            self._skip_names.update(skip_names)
 
         self._fake_module_classes = {}
+        self._unfaked_module_classes = {}
         self._class_modules = {}
         self._init_fake_module_classes()
 
-        self.modules_to_reload = [tempfile]
-        if modules_to_reload is not None:
-            self.modules_to_reload.extend(modules_to_reload)
+        self.modules_to_reload = modules_to_reload or []
+
+        if use_known_patches:
+            modules_to_patch = modules_to_patch or {}
+            modules_to_patch.update(get_modules_to_patch())
+            self._class_modules.update(get_classes_to_patch())
+            self._fake_module_classes.update(get_fake_module_classes())
 
         if modules_to_patch is not None:
             for name, fake_module in modules_to_patch.items():
@@ -348,11 +430,14 @@ class Patcher(object):
 
         # Attributes set by _refresh()
         self._modules = {}
+        self._skipped_modules = {}
         self._fct_modules = {}
+        self._def_functions = []
         self._open_functions = {}
         self._stubs = None
         self.fs = None
         self.fake_modules = {}
+        self.unfaked_modules = {}
         self._dyn_patcher = None
 
         # _isStale is set by tearDown(), reset by _refresh()
@@ -369,25 +454,30 @@ class Patcher(object):
             'shutil': fake_filesystem_shutil.FakeShutilModule,
             'io': fake_filesystem.FakeIoModule,
         }
-        if IS_PY2 or IS_PYPY:
-            # in Python 2 io.open, the module is referenced as _io
+        if IS_PYPY:
+            # in PyPy io.open, the module is referenced as _io
             self._fake_module_classes['_io'] = fake_filesystem.FakeIoModule
 
         # class modules maps class names against a list of modules they can
         # be contained in - this allows for alternative modules like
         # `pathlib` and `pathlib2`
+        self._class_modules['Path'] = []
         if pathlib:
-            self._class_modules['Path'] = []
-            if pathlib:
-                self._fake_module_classes[
-                    'pathlib'] = fake_pathlib.FakePathlibModule
-                self._class_modules['Path'].append('pathlib')
-            if pathlib2:
-                self._fake_module_classes[
-                    'pathlib2'] = fake_pathlib.FakePathlibModule
-                self._class_modules['Path'].append('pathlib2')
             self._fake_module_classes[
-                'Path'] = fake_pathlib.FakePathlibPathModule
+                'pathlib'] = fake_pathlib.FakePathlibModule
+            self._class_modules['Path'].append('pathlib')
+            self._unfaked_module_classes[
+                'pathlib'] = fake_pathlib.RealPathlibModule
+        if pathlib2:
+            self._fake_module_classes[
+                'pathlib2'] = fake_pathlib.FakePathlibModule
+            self._class_modules['Path'].append('pathlib2')
+            self._unfaked_module_classes[
+                'pathlib2'] = fake_pathlib.RealPathlibModule
+        self._fake_module_classes[
+            'Path'] = fake_pathlib.FakePathlibPathModule
+        self._unfaked_module_classes[
+            'Path'] = fake_pathlib.RealPathlibPathModule
         if use_scandir:
             self._fake_module_classes[
                 'scandir'] = fake_scandir.FakeScanDirModule
@@ -434,18 +524,20 @@ class Patcher(object):
                     mod.__name__ in module_names
                     or inspect.isclass(mod) and
                     mod.__module__ in self._class_modules.get(name, []))
-        except AttributeError:
+        except Exception:
             # handle cases where the module has no __name__ or __module__
-            # attribute - see #460
+            # attribute - see #460, and any other exception triggered
+            # by inspect functions
             return False
 
-    def _is_open_function(self, fct):
+    def _is_skipped_fs_module(self, mod, name, module_names):
         try:
-            return ((inspect.isfunction(fct) or
-                     inspect.isbuiltin(fct)) and
-                    fct.__name__ == 'open' and
-                    fct.__module__ == BUILTIN_MODULE)
-        except AttributeError:
+            return (inspect.ismodule(mod) and
+                    mod.__name__ in module_names)
+        except Exception:
+            # handle cases where the module has no __name__ or __module__
+            # attribute - see #460, and any other exception triggered
+            # by inspect functions
             return False
 
     def _is_fs_function(self, fct):
@@ -455,11 +547,43 @@ class Patcher(object):
                     fct.__name__ in self._fake_module_functions and
                     fct.__module__ in self._fake_module_functions[
                         fct.__name__])
-        except AttributeError:
+        except Exception:
             # handle cases where the function has no __name__ or __module__
-            # attribute
+            # attribute, or any other exception in inspect functions
             return False
 
+    def _def_values(self, item):
+        """Find default arguments that are file-system functions to be
+        patched in top-level functions and members of top-level classes."""
+        # check for module-level functions
+        try:
+            if inspect.isfunction(item):
+                if item.__defaults__:
+                    for i, d in enumerate(item.__defaults__):
+                        if self._is_fs_function(d):
+                            yield item, i, d
+            elif inspect.isclass(item):
+                # check for methods in class
+                # (nested classes are ignored for now)
+                with warnings.catch_warnings():
+                    # ignore deprecation warnings, see #542
+                    warnings.filterwarnings(
+                        'ignore',
+                        category=DeprecationWarning
+                    )
+                    for m in inspect.getmembers(item,
+                                                predicate=inspect.isfunction):
+                        m = m[1]
+                        if m.__defaults__:
+                            for i, d in enumerate(m.__defaults__):
+                                if self._is_fs_function(d):
+                                    yield m, i, d
+        except Exception:
+            # Ignore any exception, examples:
+            # ImportError: No module named '_gdbm'
+            # _DontDoThat() (see #523)
+            pass
+
     def _find_modules(self):
         """Find and cache all modules that import file system modules.
         Later, `setUp()` will stub these with the fake file system
@@ -469,15 +593,17 @@ class Patcher(object):
         module_names = list(self._fake_module_classes.keys()) + [PATH_MODULE]
         for name, module in list(sys.modules.items()):
             try:
-                if (module in self.SKIPMODULES or
-                        not inspect.ismodule(module) or
-                        module.__name__.split('.')[0] in self._skipNames):
+                if module in self.SKIPMODULES or not inspect.ismodule(module):
                     continue
-            except AttributeError:
+            except Exception:
                 # workaround for some py (part of pytest) versions
                 # where py.error has no __name__ attribute
                 # see https://github.com/pytest-dev/py/issues/73
+                # and any other exception triggered by inspect.ismodule
                 continue
+            skipped = (any([sn.startswith(module.__name__)
+                            for sn in self._skip_names]))
+            module_items = module.__dict__.copy().items()
 
             # suppress specific pytest warning - see #466
             with warnings.catch_warnings():
@@ -487,26 +613,31 @@ class Patcher(object):
                     category=DeprecationWarning,
                     module='py'
                 )
-                modules = {name: mod for name, mod in
-                           module.__dict__.copy().items()
+                modules = {name: mod for name, mod in module_items
                            if self._is_fs_module(mod, name, module_names)}
+
+            if skipped:
+                for name, mod in modules.items():
+                    self._skipped_modules.setdefault(name, set()).add(
+                        (module, mod.__name__))
+                continue
+
             for name, mod in modules.items():
-                self._modules.setdefault(name, set()).add((module,
-                                                           mod.__name__))
+                self._modules.setdefault(name, set()).add(
+                    (module, mod.__name__))
             functions = {name: fct for name, fct in
-                         module.__dict__.copy().items()
+                         module_items
                          if self._is_fs_function(fct)}
+
+            # find default arguments that are file system functions
+            for _, fct in module_items:
+                for f, i, d in self._def_values(fct):
+                    self._def_functions.append((f, i, d))
+
             for name, fct in functions.items():
                 self._fct_modules.setdefault(
                     (name, fct.__name__, fct.__module__), set()).add(module)
 
-            if IS_PY2:
-                open_fcts = {name for name, fct in
-                             module.__dict__.copy().items()
-                             if self._is_open_function(fct)}
-                for name in open_fcts:
-                    self._open_functions.setdefault(name, set()).add(module)
-
     def _refresh(self):
         """Renew the fake file system and set the _isStale flag to `False`."""
         if self._stubs is not None:
@@ -514,10 +645,15 @@ class Patcher(object):
         self._stubs = mox3_stubout.StubOutForTesting()
 
         self.fs = fake_filesystem.FakeFilesystem(patcher=self)
+        self.fs.patch_open_code = self.patch_open_code
         for name in self._fake_module_classes:
             self.fake_modules[name] = self._fake_module_classes[name](self.fs)
+            if hasattr(self.fake_modules[name], 'skip_names'):
+                self.fake_modules[name].skip_names = self._skip_names
         self.fake_modules[PATH_MODULE] = self.fake_modules['os'].path
         self.fake_open = fake_filesystem.FakeFileOpen(self.fs)
+        for name in self._unfaked_module_classes:
+            self.unfaked_modules[name] = self._unfaked_module_classes[name]()
 
         self._isStale = False
 
@@ -539,6 +675,8 @@ class Patcher(object):
             doctester.globs = self.replace_globs(doctester.globs)
 
         self.start_patching()
+        linecache.open = self.original_open
+        tokenize._builtin_open = self.original_open
 
         # the temp directory is assumed to exist at least in `tempfile1`,
         # so we create it here for convenience
@@ -548,24 +686,9 @@ class Patcher(object):
         if not self._patching:
             self._patching = True
 
-            if IS_PY2:
-                self._stubs.smart_set(builtins, 'open', self.fake_open)
-                self._stubs.smart_set(builtins, 'file', self.fake_open)
-
-            for name, modules in self._modules.items():
-                for module, attr in modules:
-                    self._stubs.smart_set(
-                        module, name, self.fake_modules[attr])
-            for (name, ft_name, ft_mod), modules in self._fct_modules.items():
-                method, mod_name = self._fake_module_functions[ft_name][ft_mod]
-                fake_module = self.fake_modules[mod_name]
-                attr = method.__get__(fake_module, fake_module.__class__)
-                for module in modules:
-                    self._stubs.smart_set(module, name, attr)
-            if IS_PY2:
-                for name, modules in self._open_functions.items():
-                    for module in modules:
-                        self._stubs.smart_set(module, name, self.fake_open)
+            self.patch_modules()
+            self.patch_functions()
+            self.patch_defaults()
 
             self._dyn_patcher = DynamicPatcher(self)
             sys.meta_path.insert(0, self._dyn_patcher)
@@ -573,6 +696,39 @@ class Patcher(object):
                 if module.__name__ in sys.modules:
                     reload(module)
 
+    def patch_functions(self):
+        for (name, ft_name, ft_mod), modules in self._fct_modules.items():
+            method, mod_name = self._fake_module_functions[ft_name][ft_mod]
+            fake_module = self.fake_modules[mod_name]
+            attr = method.__get__(fake_module, fake_module.__class__)
+            for module in modules:
+                self._stubs.smart_set(module, name, attr)
+
+    def patch_modules(self):
+        for name, modules in self._modules.items():
+            for module, attr in modules:
+                self._stubs.smart_set(
+                    module, name, self.fake_modules[attr])
+        for name, modules in self._skipped_modules.items():
+            for module, attr in modules:
+                if attr in self.unfaked_modules:
+                    self._stubs.smart_set(
+                        module, name, self.unfaked_modules[attr])
+
+    def patch_defaults(self):
+        for (fct, idx, ft) in self._def_functions:
+            method, mod_name = self._fake_module_functions[
+                ft.__name__][ft.__module__]
+            fake_module = self.fake_modules[mod_name]
+            attr = method.__get__(fake_module, fake_module.__class__)
+            new_defaults = []
+            for i, d in enumerate(fct.__defaults__):
+                if i == idx:
+                    new_defaults.append(attr)
+                else:
+                    new_defaults.append(d)
+            fct.__defaults__ = tuple(new_defaults)
+
     def replace_globs(self, globs_):
         globs = globs_.copy()
         if self._isStale:
@@ -595,9 +751,21 @@ class Patcher(object):
             self._isStale = True
             self._patching = False
             self._stubs.smart_unset_all()
+            self.unset_defaults()
             self._dyn_patcher.cleanup()
             sys.meta_path.pop(0)
 
+    def unset_defaults(self):
+        for (fct, idx, ft) in self._def_functions:
+            new_defaults = []
+            for i, d in enumerate(fct.__defaults__):
+                if i == idx:
+                    new_defaults.append(ft)
+                else:
+                    new_defaults.append(d)
+            fct.__defaults__ = tuple(new_defaults)
+        self._def_functions = []
+
     def pause(self):
         """Pause the patching of the file system modules until `resume` is
         called. After that call, all file system calls are executed in the
@@ -616,7 +784,7 @@ class Patcher(object):
         self.start_patching()
 
 
-class Pause(object):
+class Pause:
     """Simple context manager that allows to pause/resume patching the
     filesystem. Patching is paused in the context manager, and resumed after
     going out of it's scope.
@@ -647,7 +815,7 @@ class Pause(object):
         return self._fs.resume()
 
 
-class DynamicPatcher(object):
+class DynamicPatcher:
     """A file loader that replaces file system related modules by their
     fake implementation if they are loaded after calling `setUpPyfakefs()`.
     Implements the protocol needed for import hooks.
@@ -699,11 +867,6 @@ class DynamicPatcher(object):
         if self.needs_patch(fullname):
             return ModuleSpec(fullname, self)
 
-    def find_module(self, fullname, path=None):
-        """Module finder for Python 2."""
-        if self.needs_patch(fullname):
-            return self
-
     def load_module(self, fullname):
         """Replaces the module by its fake implementation."""
         sys.modules[fullname] = self.modules[fullname]
