@@ -30,17 +30,11 @@ from pyfakefs.extra_packages import pathlib, pathlib2
 from pyfakefs.fake_filesystem import is_root
 
 from pyfakefs import fake_pathlib, fake_filesystem
-from pyfakefs.helpers import text_type
-from pyfakefs.tests.test_utils import RealFsTestCase, TestCase
+from pyfakefs.tests.test_utils import RealFsTestCase
 
 is_windows = sys.platform == 'win32'
 
 
-def skip_if_pathlib_35_not_available():
-    if sys.version_info < (3, 5) and not pathlib2:
-        raise unittest.SkipTest('New in version 3.5')
-
-
 def skip_if_pathlib_36_not_available():
     if sys.version_info < (3, 6) and not pathlib2:
         raise unittest.SkipTest('Changed behavior in Python 3.6')
@@ -51,7 +45,6 @@ def skip_if_pathlib_36_is_available():
         raise unittest.SkipTest('Changed behavior in Python 3.6')
 
 
-@unittest.skipIf(pathlib is None, 'Not running without pathlib')
 class RealPathlibTestCase(RealFsTestCase):
     def __init__(self, methodName='runTest'):
         super(RealPathlibTestCase, self).__init__(methodName)
@@ -59,7 +52,7 @@ class RealPathlibTestCase(RealFsTestCase):
         self.path = None
 
     def setUp(self):
-        super(RealPathlibTestCase, self).setUp()
+        super().setUp()
         if not self.use_real_fs():
             self.pathlib = fake_pathlib.FakePathlibModule(self.filesystem)
         self.path = self.pathlib.Path
@@ -73,13 +66,21 @@ class FakePathlibInitializationTest(RealPathlibTestCase):
             self.assertTrue(isinstance(path, self.pathlib.WindowsPath))
             self.assertTrue(isinstance(path, self.pathlib.PureWindowsPath))
             self.assertTrue(self.pathlib.PurePosixPath())
-            self.assertRaises(NotImplementedError, self.pathlib.PosixPath)
+            # in fake fs, we allow to use the other OS implementation
+            if self.use_real_fs():
+                with self.assertRaises(NotImplementedError):
+                    self.pathlib.PosixPath()
+            else:
+                self.assertTrue(self.pathlib.PosixPath())
         else:
             self.assertTrue(isinstance(path, self.pathlib.PosixPath))
             self.assertTrue(isinstance(path, self.pathlib.PurePosixPath))
             self.assertTrue(self.pathlib.PureWindowsPath())
-            self.assertRaises(NotImplementedError,
-                              self.pathlib.WindowsPath)
+            if self.use_real_fs():
+                with self.assertRaises(NotImplementedError):
+                    self.pathlib.WindowsPath()
+            else:
+                self.assertTrue(self.pathlib.WindowsPath())
 
     def test_init_with_segments(self):
         """Basic initialization tests - taken from pathlib.Path documentation
@@ -234,16 +235,16 @@ class FakePathlibPurePathTest(RealPathlibTestCase):
                          self.path('etc/passwd'))
         self.assertEqual(self.path('/etc/passwd').relative_to('/'),
                          self.path('etc/passwd'))
-        self.assertRaises(ValueError, self.path('passwd').relative_to,
-                          '/usr')
+        with self.assertRaises(ValueError):
+            self.path('passwd').relative_to('/usr')
 
     def test_with_name(self):
         self.check_windows_only()
         self.assertEqual(
             self.path('c:/Downloads/pathlib.tar.gz').with_name('setup.py'),
             self.path('c:/Downloads/setup.py'))
-        self.assertRaises(ValueError, self.path('c:/').with_name,
-                          'setup.py')
+        with self.assertRaises(ValueError):
+            self.path('c:/').with_name('setup.py')
 
     def test_with_suffix(self):
         self.assertEqual(
@@ -387,8 +388,8 @@ class FakePathlibFileObjectPropertyTest(RealPathlibTestCase):
         file_stat = self.os.stat(self.file_path)
         link_stat = self.os.lstat(self.file_link_path)
         if not hasattr(os, "lchmod"):
-            self.assertRaises(NotImplementedError,
-                              self.path(self.file_link_path).lchmod, 0o444)
+            with self.assertRaises(NotImplementedError):
+                self.path(self.file_link_path).lchmod(0o444)
         else:
             self.path(self.file_link_path).lchmod(0o444)
             self.assertEqual(file_stat.st_mode, stat.S_IFREG | 0o666)
@@ -463,12 +464,6 @@ class FakePathlibFileObjectPropertyTest(RealPathlibTestCase):
         path = self.path(
             self.make_path('/path', 'to', 'file', 'this can not exist'))
         self.assertEqual(path, path.resolve())
-        # kludge to avoid https://github.com/mcmtroffaes/pathlib2/issues/45
-        if not (pathlib2 and
-                TestCase.is_windows and
-                sys.version_info < (3, 2)):
-            self.assert_raises_os_error(errno.ENOENT, path.resolve,
-                                        strict=True)
 
     def test_cwd(self):
         dir_path = self.make_path('jane')
@@ -478,7 +473,6 @@ class FakePathlibFileObjectPropertyTest(RealPathlibTestCase):
                          self.path(self.os.path.realpath(dir_path)))
 
     def test_expanduser(self):
-        skip_if_pathlib_35_not_available()
         if is_windows:
             self.assertEqual(self.path('~').expanduser(),
                              self.path(
@@ -489,15 +483,13 @@ class FakePathlibFileObjectPropertyTest(RealPathlibTestCase):
                              self.path(os.environ['HOME']))
 
     def test_home(self):
-        skip_if_pathlib_35_not_available()
         if is_windows:
-            self.assertEqual(self.path.home(),
-                             self.path(
-                                 os.environ['USERPROFILE'].replace('\\',
-                                                                   '/')))
+            self.assertEqual(self.path(
+                os.environ['USERPROFILE'].replace('\\', '/')),
+                self.path.home())
         else:
-            self.assertEqual(self.path.home(),
-                             self.path(os.environ['HOME']))
+            self.assertEqual(self.path(os.environ['HOME']),
+                             self.path.home())
 
 
 class RealPathlibFileObjectPropertyTest(FakePathlibFileObjectPropertyTest):
@@ -527,22 +519,18 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
 
     def test_open(self):
         self.create_dir(self.make_path('foo'))
-        self.assertRaises(IOError,
-                          self.path(self.make_path('foo', 'bar.txt')).open)
+        with self.assertRaises(OSError):
+            self.path(self.make_path('foo', 'bar.txt')).open()
         self.path(self.make_path('foo', 'bar.txt')).open('w').close()
         self.assertTrue(
             self.os.path.exists(self.make_path('foo', 'bar.txt')))
 
     def test_read_text(self):
-        skip_if_pathlib_35_not_available()
         self.create_file(self.make_path('text_file'), contents='foo')
         file_path = self.path(self.make_path('text_file'))
         self.assertEqual(file_path.read_text(), 'foo')
 
-    @unittest.skipIf(sys.version_info < (3, 0),
-                     'Python 3 specific string handling')
     def test_read_text_with_encoding(self):
-        skip_if_pathlib_35_not_available()
         self.create_file(self.make_path('text_file'),
                          contents='ерунда', encoding='cyrillic')
         file_path = self.path(self.make_path('text_file'))
@@ -550,17 +538,13 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
                          'ерунда')
 
     def test_write_text(self):
-        skip_if_pathlib_35_not_available()
         path_name = self.make_path('text_file')
         file_path = self.path(path_name)
-        file_path.write_text(text_type('foo'))
+        file_path.write_text(str('foo'))
         self.assertTrue(self.os.path.exists(path_name))
         self.check_contents(path_name, 'foo')
 
-    @unittest.skipIf(sys.version_info < (3, 0),
-                     'Python 3 specific string handling')
     def test_write_text_with_encoding(self):
-        skip_if_pathlib_35_not_available()
         path_name = self.make_path('text_file')
         file_path = self.path(path_name)
         file_path.write_text('ανοησίες', encoding='greek')
@@ -568,14 +552,12 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
         self.check_contents(path_name, 'ανοησίες'.encode('greek'))
 
     def test_read_bytes(self):
-        skip_if_pathlib_35_not_available()
         path_name = self.make_path('binary_file')
         self.create_file(path_name, contents=b'Binary file contents')
         file_path = self.path(path_name)
         self.assertEqual(file_path.read_bytes(), b'Binary file contents')
 
     def test_write_bytes(self):
-        skip_if_pathlib_35_not_available()
         path_name = self.make_path('binary_file')
         file_path = self.path(path_name)
         file_path.write_bytes(b'Binary file contents')
@@ -590,8 +572,6 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
         self.assertFalse(self.os.path.exists(file_name))
         self.check_contents(new_file_name, 'test')
 
-    @unittest.skipIf(sys.version_info < (3, 3),
-                     'Path.replace() available since Python 3.3')
     def test_replace(self):
         self.create_file(self.make_path('foo', 'bar.txt'), contents='test')
         self.create_file(self.make_path('bar', 'old.txt'),
@@ -616,6 +596,7 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
         self.check_contents(file_name, '')
         self.assertTrue(self.os.stat(file_name).st_mode,
                         stat.S_IFREG | 0o444)
+        self.os.chmod(file_name, mode=0o666)
 
     def test_touch_existing(self):
         file_name = self.make_path('foo', 'bar.txt')
@@ -627,19 +608,19 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
         self.check_contents(file_name, 'test')
 
     def test_samefile(self):
-        skip_if_pathlib_35_not_available()
         file_name = self.make_path('foo', 'bar.txt')
         self.create_file(file_name)
         file_name2 = self.make_path('foo', 'baz.txt')
         self.create_file(file_name2)
-        self.assertRaises(OSError,
-                          self.path(
-                              self.make_path('foo', 'other')).samefile,
-                          self.make_path('foo', 'other.txt'))
+        with self.assertRaises(OSError):
+            self.path(self.make_path('foo', 'other')).samefile(
+                self.make_path('foo', 'other.txt'))
         path = self.path(file_name)
         other_name = self.make_path('foo', 'other.txt')
-        self.assertRaises(OSError, path.samefile, other_name)
-        self.assertRaises(OSError, path.samefile, self.path(other_name))
+        with self.assertRaises(OSError):
+            path.samefile(other_name)
+        with self.assertRaises(OSError):
+            path.samefile(self.path(other_name))
         self.assertFalse(path.samefile(file_name2))
         self.assertFalse(path.samefile(self.path(file_name2)))
         self.assertTrue(
@@ -671,7 +652,6 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
                                     self.path(dir_name).mkdir)
 
     def test_mkdir_exist_ok(self):
-        skip_if_pathlib_35_not_available()
         dir_name = self.make_path('foo', 'bar')
         self.create_dir(dir_name)
         self.path(dir_name).mkdir(exist_ok=True)
@@ -688,7 +668,8 @@ class FakePathlibPathFileOperationTest(RealPathlibTestCase):
         self.assertFalse(self.os.path.exists(dir_name))
         self.assertTrue(self.os.path.exists(self.make_path('foo')))
         self.create_file(self.make_path('foo', 'baz'))
-        self.assertRaises(OSError, self.path(self.make_path('foo')).rmdir)
+        with self.assertRaises(OSError):
+            self.path(self.make_path('foo')).rmdir()
         self.assertTrue(self.os.path.exists(self.make_path('foo')))
 
     def test_iterdir(self):
@@ -993,11 +974,18 @@ class FakePathlibUsageInOsFunctionsTest(RealPathlibTestCase):
     def test_utime(self):
         path = self.make_path('some_file')
         self.create_file(path, contents='test')
-        self.os.utime(self.path(path), (1, 2))
+        self.os.utime(self.path(path), times=(1, 2))
         st = self.os.stat(path)
         self.assertEqual(1, st.st_atime)
         self.assertEqual(2, st.st_mtime)
 
+    def test_truncate(self):
+        path = self.make_path('some_file')
+        self.create_file(path, contents='test_test')
+        self.os.truncate(self.path(path), length=4)
+        st = self.os.stat(path)
+        self.assertEqual(4, st.st_size)
+
 
 class RealPathlibUsageInOsFunctionsTest(FakePathlibUsageInOsFunctionsTest):
     def use_real_fs(self):
@@ -1064,5 +1052,4 @@ class FakeFilesystemPathLikeObjectTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    if pathlib:
-        unittest.main()
+    unittest.main()
